@@ -172,20 +172,20 @@ void BayonneAudio::play(const char **files, Mode m)
 	if(isOpen())
 		AudioStream::close();
 
-	list = NULL;
-
 	if(!fn)
 		return;
 
+	list = files;
 	mode = m;
 	AudioStream::open(fn, m, framing);
 	if(!isOpen())
+	{
+		list = NULL;
 		return;
+	}
 
 	if(offset)
 		position(offset);
-
-	list = files;
 }
 
 char *BayonneAudio::getContinuation(void)
@@ -224,23 +224,27 @@ void BayonneAudio::record(const char *fname, Mode m, const char*annotation)
 	case modeCreate:
 		remove(fname);
 		create(fname, &info, false, framing);
-		return;
+		break;
 	case modeWrite:
 		open(fname, mode, framing);		
 		if(isOpen() && offset)
 			position(offset);
-		return;
+		break;
 	case modeAppend:
 		open(fname, modeWrite, framing);
 		if(isOpen())
 			setPosition();
-		return;
+		break;
 	}
 }
 
 void BayonneAudio::cleanup(void)
 {
-	if(isOpen())
-		close();
+	time_t now;
+
+	if(!isOpen())
+		return;
+
+	close();
 }
 
