@@ -716,6 +716,7 @@ void Driver::run(void)
 	char *local_uri = NULL;
 	sdp_message_t *remote_sdp, *local_sdp;
 	sdp_connection_t *conn;
+	osip_content_type_t *ct;
 	sdp_media_t *remote_media, *local_media, *audio_media;
 	char *tmp, *audio_payload;
 	const char *data_rtpmap;
@@ -788,6 +789,107 @@ void Driver::run(void)
 		case EXOSIP_REGISTRATION_TERMINATED:
 			slog.info("sip registration terminated");
 			break;	
+		case EXOSIP_CALL_MESSAGE_NEW:
+			if(!sevent->request)
+				break;
+			if(!MSG_IS_INFO(sevent->request))
+				break;
+			session = getCall(sevent->cid);
+			if(!session)
+			{
+				slog.warn("sip: got info for non-existant call");
+				goto killit;
+			}
+			ct = sevent->request->content_type;
+			if(!ct || !ct->type || !ct->subtype)
+				break;
+
+			if(stricmp(ct->type, "application") || stricmp(ct->subtype, "dtmf-relay"))
+				break;
+
+			mbody = NULL;
+			osip_message_get_body(sevent->request, 0, &mbody);
+			line=mbody->body;
+			digit[0] = 0;
+			duration[0] = 0;
+			while(line && *line)
+			{
+				k = v = line;
+				while(*v && *v != '=' && *v != '\r' && *v != '\n')
+					++v;
+
+				if(*v != '=')
+					break;
+
+				++v;
+
+				while(*v == ' ' || *v == '\b')
+					++v;
+
+				if(!*v || *v == '\r' || *v == '\n')
+					break;
+
+				l = v;
+				while(*l && *l != ' ' && *l != '\r' && *l != '\n')
+					++l;
+
+				if(!*l)
+					break;
+
+				if(!strnicmp(k, "signal", 6))
+					strncpy(digit, v, l-v);
+				else if(!strnicmp(k, "duration", 8))
+					strncpy(duration, v, l-v);
+
+				++l;
+				while(*l == ' ' || *l == '\n' || *l == '\r')
+					++l;
+
+				line = l;
+				if(!*line)
+					break;
+			}
+
+			msg = NULL;
+			eXosip_lock();
+			eXosip_call_build_answer(sevent->tid, 200, &msg);
+			eXosip_call_send_answer(sevent->tid, 200, msg);
+			eXosip_unlock();
+
+			if(!digit[0] || !duration[0])
+				break;
+
+			memset(&event, 0, sizeof(event));
+			event.id = DTMF_KEYUP;
+			session->dtmf_inband = false;
+			switch(*digit)
+			{
+                        case '0':
+                        case '1':
+                        case '2':
+                        case '3':
+                        case '4':
+                        case '5':
+                        case '6':
+                        case '7':
+                        case '8':
+                        case '9':
+                                event.dtmf.digit = *digit - '0';
+                                event.dtmf.duration = atoi(duration);
+				session->postEvent(&event);
+				break;
+			case '*':
+                                event.dtmf.digit = 10;
+                                event.dtmf.duration = atoi(duration);
+                                session->postEvent(&event);
+                                break;
+                        case '#':
+                                event.dtmf.digit = 11;
+                                event.dtmf.duration = atoi(duration);
+                                session->postEvent(&event);
+                                break;
+			}
+			break;				
 		case EXOSIP_CALL_RELEASED:
                         session = getCall(sevent->cid);
                         if(session)
