@@ -825,7 +825,12 @@ void Runtime::process(void)
 		fclose(fp);
 	}
 
-	snprintf(buf, sizeof(buf), "%s/bayonne.ctrl", keypaths.getLast("runfiles"));
+	if(Bayonne::provision_gateway)
+		snprintf(buf, sizeof(buf), "%s/gateway.ctrl",
+			keypaths.getLast("runfiles"));
+	else			
+		snprintf(buf, sizeof(buf), "%s/bayonne.ctrl", 
+			keypaths.getLast("runfiles"));
 	slog.debug("fifo=%s", buf);
 	remove(buf);
 	mkfifo(buf, 0770);
